-----------------------------------------------------------------------------------------------
-- Client Lua Script for BucketList
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
require "ChatSystemLib"
 
-----------------------------------------------------------------------------------------------
-- BucketList Module Definition
-----------------------------------------------------------------------------------------------
local BucketList = {} 
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999
local kcrSelectedText = ApolloColor.new("UI_BtnTextHoloPressedFlyby")
local kcrNormalText = ApolloColor.new("UI_BtnTextHoloNormal")
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function BucketList:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here
	o.tItems = {} -- keep track of all the list items
	o.wndSelectedListItem = nil -- keep track of which list item is currently selected

    return o
end

function BucketList:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end

function BucketList:OnSave(eLevel)
	if (eLevel ~= GameLib.CodeEnumAddonSaveLevel.Character) then
		return
	end

	tSave = { }
	tBucketList = tBucketList
	
	tSave = tBucketList
	return tSave
	
end

function BucketList:OnRestore(eLevel,tSavedData)
	if tSavedData then
		tBucketList = tSavedData
	end
end	
 

-----------------------------------------------------------------------------------------------
-- BucketList OnLoad
-----------------------------------------------------------------------------------------------
function BucketList:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("BucketList.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- BucketList OnDocLoaded
-----------------------------------------------------------------------------------------------
function BucketList:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "BucketListForm", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		
		-- item list
		self.wndItemList = self.wndMain:FindChild("ItemList")
		self.wndAddedItemText = self.wndMain:FindChild("NewEntryText")
		
		
	    self.wndMain:Show(false, true)
				
		--new item box
		
	

		-- if the xmlDoc is no longer needed, you should set it to nil
		-- self.xmlDoc = nil
		
		-- Register handlers for events, slash commands and timer, etc.
		-- e.g. Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)
		Apollo.RegisterSlashCommand("bucket", "OnBucketListOn", self)
		Apollo.RegisterEventHandler("WindowManagementReady", "OnWindowManagementReady",self)
		Apollo.RegisterEventHandler("WindowManagementReady", 			"OnWindowManagementReady", self)
 
		
		ChatSystemLib.PostOnChannel(2,"BucketList: BucketList loaded.\nUse /bucket to view/update your list")


		-- Do additional Addon initialization here
		if tBucketList then
			tBucketList = tBucketList
		else
		end
	end
end

-----------------------------------------------------------------------------------------------
-- BucketList Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here

-- on SlashCommand "/bucket"
function BucketList:OnBucketListOn()
	self.wndMain:Invoke() -- show the window
	
	if not tBucketList then
		tBucketList = { }
	else
		tBucketList = tBucketList
	end
	-- populate the item list
	self:PopulateItemList(tBucketList)
end




-----------------------------------------------------------------------------------------------
-- BucketListForm Functions
-----------------------------------------------------------------------------------------------
-- when the OK button is clicked
function BucketList:OnOK()
	self.wndMain:Close() -- hide the window
end

-- when the Cancel button is clicked
function BucketList:OnCancel()
	self.wndMain:Close() -- hide the window
end

-- when item is removed from the list
function BucketList:OnDeleteItem( wndHandler, wndControl, eMouseButton )
	local wnd = wndControl:GetParent()	
	deadItem = wnd:FindChild("Text"):GetText()
	for idx, item in pairs(tBucketList) do
		if tBucketList[idx].BucketListItem == deadItem then
			deadItemMatch = true
			table.remove(tBucketList,idx)
		end
	end
	self:PopulateItemList(tBucketList)
end
	

function BucketList:OnWindowManagementReady()
	Event_FireGenericEvent("WindowManagementAdd", {wnd = self.wndMain, strName = "Bucket List"})
end

function BucketList:OnAddNewItem( wndHandler, wndControl, eMouseButton )
	local strNewBLItemText = self.wndAddedItemText:GetText()
	tBucketList[#tBucketList + 1] = {["BucketListItem"] = strNewBLItemText, ["Status"] = ''}
	self.wndAddedItemText:SetText("Enter new BucketList Items here")
	self:PopulateItemList(tBucketList)
end

function BucketList:OnListItemChecked( wndHandler, wndControl, eMouseButton )
	self.wndSibling = wndControl:GetParent():FindChild("Text")
	strItemChecked = self.wndSibling:GetText()
	itemChecked = self.wndSibling:GetText()
	for idx, items in pairs(tBucketList) do
		if strItemChecked == tBucketList[idx].BucketListItem then
			tBucketList[idx].Status = true
			itemStatus = tBucketList[idx].Status
		end
	end

end

-----------------------------------------------------------------------------------------------
-- ItemList Functions
-----------------------------------------------------------------------------------------------
-- populate item list
function BucketList:PopulateItemList(tBucketList)
	-- make sure the item list is empty to start with
	self.tListWnd = self.wndItemList:GetChildren()
	self:DestroyItemList(self.tListWnd)
	
    -- add 20 items
	for i = 1,#tBucketList do
		if tBucketList[i].BucketListItem then
        	self:AddItem(tBucketList[i].BucketListItem)
			tItemAdded = {tBucketList[i].BucketListItem}
		end
	end
	
	-- now all the item are added, call ArrangeChildrenVert to list out the list items vertically
	self.wndItemList:ArrangeChildrenVert()
end

-- clear the item list
function BucketList:DestroyItemList(tListWnd)
	-- destroy all the wnd inside the list
	for idx,wnd in ipairs(tListWnd) do
		wnd:Destroy()
	end

	-- clear the list item array
	self.tItems = {}
	self.tListWnd = { }
	self.wndSelectedListItem = nil
end

-- add an item into the item list
function BucketList:AddItem(i)
	-- load the window item for the list item
	local wnd = Apollo.LoadForm(self.xmlDoc, "ListItem", self.wndItemList, self)
	
	-- keep track of the window item created
	self.tItems[i] = wnd
	
	-- give it a piece of data to refer to 
	local wndItemText = wnd:FindChild("Text")
	if wndItemText then -- make sure the text wnd exist
		wndItemText:SetText(i) -- set the item wnd's text to BucketList[i]
		wndItemText:SetTextColor(kcrNormalText)
	end
	wnd:SetData(i)
	for idx, item in pairs(tBucketList) do
		if tBucketList[idx].BucketListItem == i then
			strStatus = tBucketList[idx].Status
			if strStatus == true then
				wndItemText:GetParent():FindChild("Button"):SetCheck(true)
			else wndItemText:GetParent():FindChild("Button"):SetCheck(false)
			end
		end
	end
end

-- when a list item is selected
function BucketList:OnListItemSelected(wndHandler, wndControl)
    -- make sure the wndControl is valid
    if wndHandler ~= wndControl then
        return
    end
    
    -- change the old item's text color back to normal color
    local wndItemText
    if self.wndSelectedListItem ~= nil then
        wndItemText = self.wndSelectedListItem:FindChild("Text")
        wndItemText:SetTextColor(kcrNormalText)
    end
    
	-- wndControl is the item selected - change its color to selected
	self.wndSelectedListItem = wndControl
	wndItemText = self.wndSelectedListItem:FindChild("Text")
    wndItemText:SetTextColor(kcrSelectedText)
    --Don't need to print to debug
	--Print( "item " ..  self.wndSelectedListItem:GetData() .. " is selected.")
end


-----------------------------------------------------------------------------------------------
-- BucketList Instance
-----------------------------------------------------------------------------------------------
local BucketListInst = BucketList:new()
BucketListInst:Init()
